from AccessGrid.hosting.SOAPInterface import SOAPIWrapper
from AccessGrid.Descriptions import CreateStreamDescription

class AGNetworkServiceIW(SOAPIWrapper):
    """
    SOAP interface for AGNetworService.
    """

    def __init__(self, url, faultHandler = None):
        SOAPIWrapper.__init__(self, url, faultHandler)
        
    def Transform(self, streamList):
        '''
        Method to transform a set of streams.

        @param streamList: A list of stream descriptions to transform.
        @type streamList: [Description.StreamDescription]
        @return: A list of stream descriptions as result from transformation. 
        @rtype: [Description.StreamDescription]
        '''
        
        streams = self.proxy.Transform(streamList)
        retval = []

        if not streams or len(streams) < 1:
            return []
        
        for stream in streams:
            retval.append(CreateStreamDescription(stream))
           
        return retval

    def StopTransform(self, streamList):
        '''
        Stop transformation.
        
        @param streamList: A set of stream descriptions of streams we want to stop transforming.
        @type streamList: [Description.StreamDescription]
        '''
        
        return self.proxy.StopTransform(streamList)
