#!/bin/sh
#
# install.sh
#
# Install packages necessary for AG2.  This script attempts to ensure
# that the necessary packages are installed and up to date. 
#
# Note: It currently makes only minimal effort to handle failures!
#

#################################################################
# Minimal checking of prerequisites begins here
#################################################################
PREREQS=0;	# 0=satisfied, else missing

PY24=`ls -d /var/db/pkg/python-2.4*`
if [ $? -ne 0 ]; then
  echo "Can't find python-2.4. Please install it before proceeding"
  PREREQS=`expr ${PREREQS} + 1`
fi

WXMOZ=`ls -d /var/db/pkg/wxmozilla-0.5*`
if [ $? -ne 0 ]; then
  echo "Can't find wxmozilla-0.5. Please install it before proceeding"
  PREREQS=`expr ${PREREQS} + 1`
fi

WXGTK2=`ls -d /var/db/pkg/wxgtk2*`
if [ $? -ne 0 ]; then
  echo "Can't find wxgtk2. Please install it before proceeding"
  PREREQS=`expr ${PREREQS} + 1`
fi

BSDDB=`ls -d /var/db/pkg/py24-bsddb3*`
if [ $? -ne 0 ]; then
  echo "Can't find py-bsddb. Please install it before proceeding"
  PREREQS=`expr ${PREREQS} + 1`
fi

if [ ${PREREQS} -ne 0 ]; then
  echo
  echo "${PREREQS} prerequisite(s) not satisfied."
  echo "Please install missing prerequisite(s) named above before proceeding."
  exit ${PREREQS}
fi


#################################################################
# Install processing begins here
#################################################################

# Inform user what will be done and prompt to continue
#
echo
echo "This script installs the Slackware packages for the Access Grid software. " 
echo "Continue ? [y/n]"
read response
if [ "$response" != 'y' ] && [ "$response" != "Y" ]
then
   exit 5
fi


#
# Define packages
# To update this script to install newer versions of packages
# change the variables defined here
#

export GLOBUS="globus-data-management-client"
export GLOBUS_VER="2.4.3"

export AG="AccessGrid"
export AG_VER=VER
 

#
# Install the specified package and version
# 
Install() 
{
  export package="$1"
  export package_ver="$2"
  export package_tgz="$package-$package_ver.tgz"

  pkg_info ${package}-${package_ver} 1>/dev/null 2>/dev/null
  if [ $? -eq 0 ]; then
    echo
    echo "${package}-${package_ver} is already installed!"
    echo "Please pkg_delete it first if you really want to install this new one"
    return
  fi
  pkg_add $package_tgz

  if [ ! $? -eq 0 ]; then
    echo "Trouble installing ${package_tgz}, exiting now !!!"
    exit 1
  fi
}

#
# Install globus package
#
echo ""
echo "***********************************************"
echo "Installing ${GLOBUS}-${GLOBUS_VER} " 
echo "***********************************************"
Install $GLOBUS $GLOBUS_VER
echo ""

unset GLOBUS_PATH
unset GLOBUS_LOCATION
. /etc/profile.d/globus.sh

#
# Install AG package
#
echo ""
echo "***********************************************"
echo "Installing Access Grid package "
echo "***********************************************"
Install $AG $AG_VER
echo ""

#
# Any weird stuff for Shared apps?
#

echo ""
echo "Installation finished."
echo ""

exit 0

