############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

from pyGlobus.ftpClient import FtpClient
from pyGlobus.ftpClientAttr   import HandleAttr
from pyGlobus.ftpClientAttr   import OperationAttr
from pyGlobus import ftpControl
from pyGlobus import ftpClientc
from pyGlobus.util import Buffer
from pyGlobus.util import Abstime
import unittest,ConfigParser, sys, traceback
from pyGlobus.util import GlobusException
from threading import *
from exceptions import Exception
import time


g_cond = 0
g_file = None
g_oldCbHandle = None
g_newCbHandle = None
g_ftpClient = None
g_buffer = None
#The config file is ~/pyGlobus/test/config.py
config_file = 'config.py'
config = None # ConfigParser object



class FtpClientTestCasePutAndGet(unittest.TestCase):
    "Test case for the FtpClient Module"

    def setUp(self):
        self._handleAttr = HandleAttr()
        #self._plugin = RestartMarkerPlugin(beginCB, markerCB, completeCB, None)
        #self._handleAttr.add_plugin(self._plugin)
        self._ftpClient = FtpClient(self._handleAttr)
        self._old_cb = None
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
        return
    
    def tearDown(self):
        global  g_oldCbHandle, g_newCbHandle
        print "in tearDown"
        time.sleep(2)
        try:
            del self._ftpClient
        except GlobusException, ex:
            print ex.args
            
        
        print "leaving tearDown"
        return
    

    def data_func_get(self,cv, handle, buffer, bufHandle, bufLen, offset, eof, error):
        global g_dest
        g_dest.write(buffer)
        if not eof:
            try:
                cbHandle = self._ftpClient.register_read(self._buffer, self.data_func_get, 0)
                if not self._old_cb == None:
                    self._ftpClient.free_callback(self._old_cb)
                self._old_cb = cbHandle
            
            except Exception, e:
                traceback.print_exception()
                sys.exit(-1)
        return

    def checkFtpClientGet(self):
        print " Unittest for the ftpClientGetTestCase"
        global g_dest, g_cond
        
        dest = config.get("ftpClient", "dest")
        url = config.get("ftpClient", "srcUrl")
        print "Destination File: ", dest
        print "Retrieving URL: ", url
        
        g_dest = open(dest, "w")
        condV = Condition(Lock())
        try:
            condV.acquire()
            g_cond = 0
            self._ftpClient.get(url, done_func, condV)
            self._buffer = Buffer(64*1024)
            cbHandle = self._ftpClient.register_read(self._buffer, self.data_func_get, 0)
            
        except GlobusException, ex:
            print ex.args
            sys.exit(-1)
        
        print "waiting on done_func"
        while g_cond == 0:
            condV.wait()

        condV.release()
        g_dest.close()
        try:
            self._ftpClient.free_callback(cbHandle)
        except GlobusException, ex:
            print ex.args

    def data_func_put(self,cv, handle, buffer, bufHandle, bufLen, offset, eof, error):
        global g_file
        if not eof:
        
            try:
                size = 1024 *64
                self._buffer = g_file.read(size)
                cbHandle = self._ftpClient.register_write(self._buffer,
                                                          len(self._buffer),
                                                          bufLen + offset,
                                                          self._buffer == "",
                                                          self.data_func_put, 0)
                                            
                if not self._old_cb == None:
                    self._ftpClient.free_callback(self._old_cb)
                self._old_cb = cbHandle
            except Exception, e:
                traceback.print_exception()
                sys.exit(-1)
        return
    
    def checkFtpClientPut(self):
        print "Unittest for the FtpClientPut case"
        global g_file, g_cond 
        print "g_file is:", g_file
        src = config.get("ftpClient", "src")
        url = config.get("ftpClient", "url")
        print "Src file", src
        print "url: ", url
        g_file = open(src, "r")
        condV = Condition(Lock())
        try:
            
            
            size = 64*1024
            oset = 0
            self._buffer = g_file.read(size)
            condV.acquire()
            g_cond = 0
            self._ftpClient.put(url,done_func, condV)
            cbHandle = self._ftpClient.register_write(self._buffer,
                                                      len(self._buffer),
                                                      oset, self._buffer == "",
                                                      self.data_func_put, 0)
            
            
        except GlobusException, ex:
            print ex.args
            sys.exit(-1)

        print "waiting on done_func"
        while g_cond == 0:
            condV.wait()
        print "done waiting"
        condV.release()
    
        self._ftpClient.free_callback(cbHandle)    
        g_file.close()
        print "DONE"
        
   
        
class FtpClientTestCaseOperations(unittest.TestCase):
    "Test case for the FtpClient Module"

    def setUp(self):
        global config
        self._handleAttr = HandleAttr()
        self._ftpClient = FtpClient(self._handleAttr)
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
        return
    
    def tearDown(self):
        global g_ftpClient
        print "in tearDown"
        time.sleep(2)
        try:
            del self._ftpClient
        except GlobusException, ex:
            print ex.args
        g_ftpClient = None
        print "leaving tearDown"
        return

    
    def checkFtpClientDelete(self):
        print "Unittest for the ftpClientDelete Test Case"
        global g_dest, g_ftpClient, g_buffer, g_newCbHandle, g_cond
        url = config.get("ftpClient", "url")
        condV = Condition(Lock())
        print url
        try:
           
           opAttr = OperationAttr()
           condV.acquire()
           g_cond = 0
           print "calling delete in test_ftp"
           self._ftpClient.delete(url,opAttr, done_func, condV)
           print "return from delete call"
           
        except GlobusException, ex:
            print "globus exception"
            print ex.args

        print "waiting on callback"
        condV.wait()
        condV.release()

    def checkFtpClientSize(self):
        print "Unittest for size"
        global g_file, g_ftpClient, g_newCbHandle, g_cond
        f = open("/tmp/test-file", "w+")
        url = config.get("ftpClient", "srcUrl")
        condV = Condition(Lock())
        try:
            
            opAttr = OperationAttr()
            opAttr.set_mode(ftpControl.MODE_EXTENDED_BLOCK)
            condV.acquire()
            g_cond = 0
            size =  self._ftpClient.size(url,done_func, condV, opAttr)
              
        except GlobusException, ex:
            raise
            #sys.exit(-1)

        print "waiting for callbacks"
        condV.wait()
        condV.release()
        
        print "size is: ", size.getLong()
        del size
        print "DONE"

    def checkFtpClientMtime(self):
    
        print "Unittest for modification time"
        global g_file, g_ftpClient, g_newCbHandle, g_cond 
        f = open("/tmp/test-file", "w+")
        url = config.get("ftpClient", "srcUrl")
        condV = Condition(Lock())
        try:
            
            opAttr = OperationAttr()
            opAttr.set_mode(ftpControl.MODE_EXTENDED_BLOCK)
            g_cond = 0
            condV.acquire()
            mt =  self._ftpClient.modification_time(url, done_func, condV, opAttr)
            
            print "waiting for callback"
            while g_cond == 0:
                condV.wait()
            
            condV.release()
        except GlobusException, ex:
            raise
        time.sleep(2)
        print "Modification_time is: ", time.ctime(mt.getSecs())
        print "DONE"    


    def checkFtpClientExists(self):
        print "Unittest for exists"
        global  g_file, g_ftpClient, g_newCbHandle, g_cond
        f = open("/tmp/test-file", "w+")
        bad_url = config.get("ftpClient", "badUrl")
        condV = Condition(Lock())
        try:
            
            opAttr = OperationAttr()
            opAttr.set_mode(ftpControl.MODE_EXTENDED_BLOCK)
            condV.acquire()
            g_cond = 0

            self._ftpClient.exists(bad_url, done_func_error_expected, condV, opAttr)
            

            while g_cond == 0:
                condV.wait()
                condV.release()
        except GlobusException, ex:
            raise
        print "DONE"
        
    def checkFtpClientAbort(self):
        """This has a time.sleep, to give the ftp handle a chance to connect, before the
              operation is aborted.
              Otherwise this error is printed out 
              globus_ftp_control_force_close: Handle is not connected
              If this happens the complete callback is never called which means the function will hang
              because the condition variables are not notified.
              """
        global g_cond, g_ftpClient
        print "Unittest for ftpClient abort test"
        src = config.get("ftpClient", "src")
        url = config.get("ftpClient", "url") 
        
        condV = Condition(Lock())
        try:
            
            condV.acquire()
            g_cond = 0
            self._ftpClient.put(url, done_func_error_expected, condV)
            time.sleep(1)
            self._ftpClient.abort()
            
        except GlobusException, ex:
            print ex
            
        print "waiting on callbacks"
        while g_cond == 0:
            condV.wait()
            condV.release()
        
             
    
        
class FtpClientTestCaseThirdPartyTransfer(unittest.TestCase):
    "Test case for the FtpClient Module"

    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
        return
    
    
    def tearDown(self):
        global g_ftpClient
        print "in tearDown"
        time.sleep(2)
        try:
            del g_ftpClient
        except GlobusException, ex:
            print ex.args

        print "leaving tearDown"
        return
    
    def checkThirdPartyTransfer(self):
        print " Unittest for the ftpClient3rdpartyTestCase"
        global g_cond, g_dest, g_ftpClient, g_buffer, g_newCbHandle   
        g_cond = 0

        srcUrl = config.get("ftpClient", "srcUrl")
        destUrl = config.get("ftpClient", "destUrl")
        print "From: ", srcUrl, " To: ", destUrl
        condV = Condition(Lock())
        try:
            handleAttr = HandleAttr()
            #plugin = RestartMarkerPlugin(beginCB, markerCB,
            #                              completeCB, None)
            #handleAttr.add_plugin(plugin)
            g_ftpClient = FtpClient(handleAttr)
            
            opAttr = OperationAttr()
            condV.acquire()
            cb = g_ftpClient.third_party_transfer(srcUrl,destUrl,
                                                  done_func, condV)
            
        except GlobusException, ex:
            print "globus exception"
            print ex.args

        while g_cond == 0:
            print "waiting on callback"
            condV.wait()
            condV.release()
        
        print "DONE"
        


        
def done_func_error_expected(cv, handle, error):
    global g_cond
    if not error == "NONE":
        print error
        print "TEST PASSED"
    else:
        print "TEST FAILED"
    cv.acquire()
    g_cond = 1
    cv.notify()
    cv.release()    
    return

def done_func(cv, handle, error):
    global g_cond
    
    print "inside done func"
    if not error == "NONE":
        print "The following error occured:", error
    cv.acquire()
    g_cond = 1
    cv.notify()
    cv.release()
    return

def done_funcPT(cv, handle, error):
    global g_cond
    print "done_funcPT"
    cv.acquire()
    if not error == "NONE":
        cv.notify()
        cv.release()
        print "The following error occured:",  error
        raise GlobusException, error
    else:
        g_cond = 1
    cv.notify()
    cv.release()
    return





def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(FtpClientTestCasePutAndGet, "check"))
    suite.addTest(unittest.makeSuite(FtpClientTestCaseOperations, "check"))
    suite.addTest(unittest.makeSuite(FtpClientTestCaseThirdPartyTransfer, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
   
 
