############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
from pyGlobus.ftpClientPlugin import PerformanceMarkerPlugin
from pyGlobus.ftpClientPlugin import DebugPlugin
from pyGlobus.ftpClientPlugin import RestartMarkerPlugin
from pyGlobus.ftpClientPlugin import RestartMarker
from pyGlobus.ftpClientPlugin import ThroughputPerfPlugin
from pyGlobus.ftpClient import EasyFtpClient
from pyGlobus.ftpClientAttr   import HandleAttr
from pyGlobus.ftpClientAttr   import OperationAttr
from pyGlobus.util import GlobusException
from pyGlobus import ftpControl
from pyGlobus import ftpClientc
import unittest,ConfigParser, sys, traceback
import time
from threading import *



config_file = 'config.py'
config = None # ConfigParser object

   
class FtpPluginRestartMarkerTestCase(unittest.TestCase):
    "Test case for the FtpPlugin Module"
    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
            
        self.__marker_str = ''
        return
    
    
    def tearDown(self):
        print "in tearDown"
        time.sleep(2)
        try:
            del self.__ftpClient
        except GlobusException, ex:
            print ex.args

        print "leaving tearDown"
        return 


    def __beginCB(self, arg, handle, srcUrl, destUrl, marker):
        ret = 0
        print "inside the beginCB"
        if self.__marker_str:
            print "\tRestart Marker String: ", self.__marker_str
            ftpClientc.restart_marker_from_string(marker, self.__marker_str)
            print "\tout"
        ret = 1
        print "leaving beginCB"
        return ret

    def __completeCB(self, arg, handle, err, url):
        print "inside the complete CB"
        if not err == "NONE":
            print "got the following error: ", err
  
        return
    
    def __marker_abort_CB(self, arg, handle, marker):
        print "inside the marker_abort_CB"
        ret, err, self.__marker_str = ftpClientc.restart_marker_to_string(marker)
        
        try:
            self.__ftpClient.abort()
        except Exception:
            print "Failed Abort"
        else:
            print "\tabort"
      
        print "\trestart marker string is:", self.__marker_str
        return

    def __markerCB(self, arg, handle, marker):
        print "inside the markerCB"
        ret, err, self.__marker_str = ftpClientc.restart_marker_to_string(marker)
        print "restart marker string is:", self.__marker_str
        return
    
    def checkRestartMarkerPlugin(self, offset=0L, endOffset=100L):
        print " Unittest for the checkRestartMarkerPlugin"   
        destUrl = config.get("ftpClient", "destUrl")
        srcUrl = config.get("ftpClient", "srcUrl")
        print "srcUrl = ", srcUrl
        print "dstUrl = ", destUrl
        _markerCB_func = self.__marker_abort_CB
        for i in range(2):
            print "-Start trans ->"
            try:
                handleAttr = HandleAttr()
                plugin = RestartMarkerPlugin(self.__beginCB,_markerCB_func,
                                             self.__completeCB, None)
                handleAttr.add_plugin(plugin)
                self.__ftpClient = EasyFtpClient(handleAttr)
                opAttr = OperationAttr()
                opAttr.set_mode(ftpControl.MODE_EXTENDED_BLOCK)
                print "mode is:",  opAttr.get_mode()
                
                self.__ftpClient.third_party_transfer(srcUrl,destUrl)
                                                      
            except GlobusException, ex:
                print "globus exception"
                print ex
            
            _markerCB_func = self.__markerCB            
        print "DONE"
        return

    def checkPartialThirdPartyTransfer(self, offset=0L, endOffset=10L):
        print " Unittest for the ftpClient3rdpartyPartialTestCase with restart marker"
        

        srcUrl = config.get("ftpClient", "srcUrl")
        destUrl = config.get("ftpClient", "destUrl")
        destUrl = destUrl + '_partial1'
        
        srcAttr = OperationAttr()
        srcAttr.set_mode(ftpControl.MODE_EXTENDED_BLOCK)
        dstAttr = OperationAttr()
        dstAttr.set_mode(ftpControl.MODE_EXTENDED_BLOCK)
        print type(dstAttr)
        marker  = RestartMarker()
        r_plugin = RestartMarkerPlugin(self.__beginCB, self.__markerCB,
                                     self.__completeCB, None)
        for i in range(3):
            
            try:
                
                handleAttr = HandleAttr()
                handleAttr.add_plugin(r_plugin)
                self.__ftpClient = EasyFtpClient(handleAttr)
                print "Into ptpt\n\tsrc(%s) dst(%s)" % (srcUrl, destUrl)
                cb = self.__ftpClient.partial_third_party_transfer(srcUrl,destUrl,
                                                                   offset, endOffset,
                                                                   srcAttr, dstAttr,
                                                                   marker)
                                                                  
                                                                   
                                                                   
                
            except GlobusException, ex:
                print "globus exception"
                print ex.args
                break
            else:
                # seem to have cleanup problem w/ 2GHz P4
                time.sleep(0.001)
                offset = endOffset
                endOffset = offset + 1
            
        print "DONE"



class FtpPluginDebugTestCase(unittest.TestCase):
    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
            
        self.__marker_str = ''
        #self.__condV   = Condition(Lock())
        self.__ftpC = None
        return
    
    
    def tearDown(self):
        print "in tearDown"
        #time.sleep(2)
        try:
            del self.__ftpClient
        except GlobusException, ex:
            print ex.args

        print "leaving tearDown"
        return 

    

    def checkDebugPlugin(self):
        print "test Debug Plugin"
        plugin = DebugPlugin(sys.stdout, "Running Unittest")
        handleAttr = HandleAttr()
        handleAttr.add_plugin(plugin)
        self.__ftpClient = EasyFtpClient(handleAttr)
        src = config.get("ftpClient", "src")
        url = config.get("ftpClient", "url")
        try:
            self.__ftpClient.put(src, url)
            
            
        except GlobusException, ex:
            print ex.args
            sys.exit(-1)

        
        
        print "DONE"
        
class FtpPluginPerformanceTestCase(unittest.TestCase):
    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
            
        self.__ftpClient = None
        return
    
    
    def tearDown(self):
        print "in tearDown"
        time.sleep(2)
        try:
            del self.__ftpClient
        except GlobusException, ex:
            print ex.args

        print "leaving tearDown"
        return 

    def __beginCB(self, arg, handle, srcUrl, destUrl, restart):
        print "inside the markerCB"
        print "srcUrl: ", srcUrl
        print "destUrl: ", destUrl
        print "restart: ", restart
        return
    
   
    def __markerCB(self, arg, handle, time_stamp, time_stamp_length, stripe_ndx, num_stripes, nbytes):
        print "inside markerCB"
        print "time_stamp: ", time_stamp
        print type(time_stamp)
        print "time_stamp: ", time.ctime(time_stamp)
        print "stripe_ndx: ", stripe_ndx
        print "num_stripes: ", num_stripes
        print "nbytes: ", nbytes
        print "leaving __markerCB"
        
    def __completeCB(self, arg, handle, error):
        print "inside the complete CB"
        print "error: ", error
        print "leaving completeCB"
        return
    

    def checkPerformancePlugin(self):
        print "test Performance Plugin"
        plugin = PerformanceMarkerPlugin(self.__beginCB, self.__markerCB,
                                      self.__completeCB, 0)
        handleAttr = HandleAttr()
        handleAttr.add_plugin(plugin)
        self.__ftpClient = EasyFtpClient(handleAttr)
        src = config.get("ftpClient", "src")
        url = config.get("ftpClient", "url")
        
        try:
            
            opAttr = OperationAttr()
            #opAttr.set_mode(ftpControl.MODE_EXTENDED_BLOCK)
            self.__ftpClient.put(src, url, opAttr)

        except GlobusException, ex:
            print ex.args
            sys.exit(-1)
            
        print "DONE"
        
class FtpPluginThroughPutTestCase(unittest.TestCase):
    """ Class that tests the ftpClientPlugin.ThroughputPerfPlugin class"""
    
    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
            
        self.__ftpClient = None
        return
    
    
    def tearDown(self):
        print "in tearDown"
        time.sleep(2)
        try:
            del self.__ftpClient
        except GlobusException, ex:
            print ex.args

        print "leaving tearDown"
        return 

    def __beginCB(self, arg, handle, srcUrl, destUrl):
        print "inside the markerCB"
        print "srcUrl: ", srcUrl
        print "destUrl: ", destUrl
        return
    
    def __stripeCB(self, arg, handle, stripe_ndx, bytes, through_put, avg_through_put):
        print "Inside stripeCB"
        print "stripe_ndx: ", stripe_ndx
        print "bytes: ", bytes
        print "through_put: ", through_put
        print "avg_through_put", avg_through_put
        print "leaving stripeCB"
        
    def __totalCB(self, arg, handle,bytes, through_put, avg_through_put):
        print "Inside totalCB"
        print "bytes: ", bytes
        print "through_put: ", through_put
        print "avg_through_put", avg_through_put
        print "leaving stripeCB"
        
    def __completeCB(self, arg, handle, success):
        print "inside the complete CB"
        print "success: ", success
        print "leaving completeCB"
        return
    

    def checkThroughputPerfPlugin(self):
        print "test ThroughPut Plugin"
        plugin = ThroughputPerfPlugin(self.__beginCB, self.__stripeCB,self.__totalCB, self.__completeCB, 0)
        handleAttr = HandleAttr()
        handleAttr.add_plugin(plugin)
        self.__ftpClient = EasyFtpClient(handleAttr)
        src = config.get("ftpClient", "src")
        url = config.get("ftpClient", "url")
        print "src: ", src
        
        self.__file = open(src, "r")
        self.__condV = Condition(Lock())
        try:
            self.__ftpClient.put(src, url)
            
        except GlobusException, ex:
            print ex.args
            sys.exit(-1)

        print "DONE"
        
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(FtpPluginRestartMarkerTestCase, "check"))
    suite.addTest(unittest.makeSuite(FtpPluginDebugTestCase, "check"))
    suite.addTest(unittest.makeSuite(FtpPluginPerformanceTestCase, "check"))
    suite.addTest(unittest.makeSuite(FtpPluginThroughPutTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
