############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

from pyGlobus import utilc
from pyGlobus.util import GlobusException
from pyGlobus.ftpControl import FtpControl
from pyGlobus.ftpControl import Parallelism
from pyGlobus.ftpControl import HostPortArray
from pyGlobus.ftpControl import HostPort
from pyGlobus.ftpControl import TcpBuffer
from pyGlobus.ftpControl import DCAU
from pyGlobus.ftpControl import FtpControlException
from pyGlobus.security import GSSCred
import unittest,ConfigParser
from pyGlobus import ftpControlc
from pyGlobus import ftpControl
from socket import gethostname,gethostbyname
import time
from pyGlobus.ftpControl import AuthInfo
config = None
config_file = 'config.py'
from threading import *
g_cond = 0

def connectCb(cv, handle, stripe_ndx, reuse, error):
    global g_cond
    print "in connectCb"
    print "error: ", error
    print "stripe_ndex:", stripe_ndx
    print "reuse: ", reuse
    cv.acquire()
    g_cond = 0
    cv.notify()
    cv.release()
    print "leaving connnectCb"

def quitCb(cv, handle, stripe_ndx, reuse, error):
    global g_cond
    print "in quitCb"
    cv.acquire()
    g_cond = 0
    cv.notify()
    cv.release()
    print "leaving quitCb"
    

class FtpControlTest__basic(unittest.TestCase):
    def setUp(self):
	global config
	if config == None:
	    config = ConfigParser.ConfigParser()
	    config.read(config_file)
	self._ftpServer = FtpControl()
	
    def tearDown(self):
	print "DONE"
	
    def check_local_send_eof(self):
	print "check_local_send_eof"
	eof = 1
	self._ftpServer.local_send_eof(eof)
			
    def check_local_parallelism(self):
	print "check_local_parallelism"
	parallel = Parallelism()
	md = ftpControl.PARALLELISM_FIXED
	parallel.set_mode(md)
        self._ftpServer.local_parallelism(parallel)
    
    def check_local_mode(self):
	"""There is a pesky valgrind UMR bug to track down in this method MKR """
	print "check_local_mode"
        #only STREAM and EXTENDED_MODE are supported for this call in the current
        #implementation
        mode = 'S'
        self._ftpServer.local_mode(mode)
	
    def check_local_tcp_buffer(self):
	print "check_local_tcp_buffer"
        tcpBuf = TcpBuffer()
        size = 100
        tcpBuf.set_fixed(size)
        self._ftpServer.local_tcp_buffer(tcpBuf)
	
    def check_local_dcau(self):
	print "check_local_dcau"
	dcau = DCAU()
	mode = ftpControl.DCAU_SELF
	dcau.set_mode(mode)
	cred = GSSCred()
	self._ftpServer.local_dcau(dcau, cred)
	
    def check_local_pbsz(self):
	print "check_local_pbsz"
        size = 100
        self._ftpServer.local_pbsz(size)
        #def response_callback(arg,handle, error, response):
        #print "in response callback"
        #def check_parallelism_init(self):
        #print "in check_parallelism_init"
        #parallel = Parallelism()
	
    def check_local_spas(self):
        print "check_local_spas"
        size = 10
        hp_array = HostPortArray(size)
	try:
	    self._ftpServer.local_spas(hp_array)
        except GlobusException, ex:
            if str(ex).count( "this function is not implemented") <= 0:
                raise FtpControlException(ex.__str__())
            else:
                print '*Expected* and got non-implemented function!: local_spas'
	
    def check_local_spor(self):
	print "check_local_spor"
        size = 10
        hp_array = HostPortArray(size)
	try:
	    hplisteners = self._ftpServer.local_spas(hp_array)  # nonblocking hopefully
	    self._ftpServer.local_spor(hplisteners)
             
	except GlobusException, ex:
            if str(ex).count( "this function is not implemented") <= 0:
                raise FtpControlException(str(ex))
            else:
                print '*Expected* and got non-implemented function!: local_spors'
	

    def implement_chek_data_connect(self):
	print "check_data_connect"
                
    def implement_check_data_write(self):
	print "check_data_write"

    def check_local_pasv(self):
	print "check_local_pasv"
        hostPort = self._ftpServer.local_pasv()
        t =  hostPort.get_info()
        print t
        
    def check_local_port(self):
	print "check_local_port"
        host = gethostbyname(gethostname())
        port = 7575
        newL = []
        hostList = host.split('.')
        for s in hostList:
            newL.append(int(s))
        print 'HostPort() on ',newL, port
        hostPort = HostPort(newL, port)
        print 'incoming local port'
        #self._ftpServer.local_port(hostPort)
        #print help(self._ftpServer.quit, 'check_local_port calling!')
        #print dir(self._ftpServer)

    def check_local_type(self):
        print "check_local_type"
        host = gethostbyname(gethostname())
        port = 7575
        newL = []
        hostList = host.split('.')
        for s in hostList:
            newL.append(int(s))
        print 'HostPort() on ',newL, port
        hostPort = HostPort(newL, port)
        print 'incoming local port'
        self._ftpServer.local_type('I')
        
    def implement_check_read_commands(self):
	print "check_read_commands"

    def implement_check_send_response(self):
	print "check_send_response"

    def implement_check_raw_command(self):
	print "check_raw_command"
    
    def implement_check_code(self):
	print "check_code"

    def implement_check_operation(self):
	print "check_operation"

    def implement_check_create_data_info(self):
	print "check_create_data_info"

    def implement_check_release_data_info(self):
	print "check_release_data_info"
    
    def implement_check_data_write_stripe(self):
	print "check_data_write_stripe"

    def implement_check_x_data_write_stripe(self):
	print "check_x_data_write_stripe"

    def implement_check_authenticate(self):
	print "check_authenticate"
	
    def implement_check_send_command(self):
	print "check_send_command"

    def implement_check_abort(self):
	print "check_abort"
	
    def check_quit(self):
        pass
    
    def implement_check_force_close(self):
	print "check_force_close"

    def check_data_connect_read(self):
        global g_cond
        print "check_data_connect_read"
        hostPort = self._ftpServer.local_pasv()
        t = hostPort.get_info()
        print t
        condV = Condition(Lock())
        g_cond = 1
        condV.acquire()
        h = self._ftpServer.data_connect_read(connectCb, condV)
        print h
        #while g_cond:
        #    print "waiting for connectCb"
        #    condV.wait()
        #condV.release()
        
    def implement_check_data_add_channels(self):
	print "check_data_add_channels"

    def implement_check_data_send_eof(self):
	print "check_data_send_eof"

    def implement_check_data_remove_channels(self):
	print "check_data_remove_channels"

    def implement_check_data_query_channels(self):
	print "check_data_query_channels"

    def implement_check_get_total_data_channels(self):
	print "check_get_total_data_channels"

    def implement_check_data_read(self):
	print "check_data_read"

    def implement_check_local_stru(self):
	print "check_local_stru"
	
class HostPortTest(unittest.TestCase):
    """There is a bug in the host port initialization function
       It takes a hostname when it should take an ip address
    """
    def check_CreateDel(self):
        print "Testing HostPort __init__ and __del__"
        host = gethostbyname(gethostname())
        port = 7575
        newL = []
        hostList = host.split('.')
        for s in hostList:
            newL.append(int(s))
        hostPort =  HostPort(newL,port)
        t = hostPort.get_info()
        print t
        del hostPort
	
class AuthInfoTest(unittest.TestCase):

    def check_CreateDel(self):
        cred = GSSCred()
        cred.acquire_cred()
        lifetime,name = cred.inquire_cred()
        dn = name.display()
        authInfo = AuthInfo(cred, 1, "mateo", "", "", dn)
        del authInfo
        
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(FtpControlTest__basic, "check"))
    suite.addTest(unittest.makeSuite(HostPortTest, "check"))
    suite.addTest(unittest.makeSuite(AuthInfoTest, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")

if __name__ == "__main__" : main()
