############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import ConfigParser, unittest
from pyGlobus.security import *
from pyGlobus.util import GlobusException
import time
"""
Unittest for the GSSContext module.
"""

config = None # ConfigParser object
g_context_client = None
g_context_server = None

class GSSContextTestCase(unittest.TestCase):
    """
       Test case for the GSSContext module.
       """
    
    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.py")
        
            
    def checkGSSContext(self):
        try:
            ctx = GSSContext()
            del ctx
            print "\nGSSContextTestCase completed\n"
        except GlobusException, ex:
            print "Exception occured during testing."
            assert 0, ex.args
        return

    def InitContext(self,cred, context, buf=None):
        #global g_ctx_blob
        print "\ncheckInitContext\n"
        
        if buf==None:
            buf = GSSBuffer()
            
        cred.acquire_cred()
        lifetime, cred_name = cred.inquire_cred()
        
        print "Cred info:  lifetime: " + str(lifetime) + "\nName: ", cred_name.display()
        t = context.init_context(cred, cred_name,buf)
        #print t
        major = t[0]
        minor = t[1]
        print "major is: ", t[0]
        print "minor is: ", t[1]
        print "len(ctx_tokens): ", len(t[2])
        #print "ctx_blob is: ", t[2]
        ctx_tokens = t[2]
        if major == gssc.GSS_S_CONTINUE_NEEDED:
            return 0, ctx_tokens
        elif major == gssc.GSS_S_COMPLETE:
            return 1, ctx_tokens
        else:
            list = gssc.display_status(major, minor)
            ex = apply(GSSContextException, list)
            raise ex
         
            
    
            
    def AcceptContext(self, cred, context, buf):
        print "checkAcceptContext\n"

        try:
            cred.acquire_cred()
            major,minor,ctx_tokens = context.accept_context(cred,buf)
            print "major is:", major
            print "minor is:", minor
            print "len(ctx_tokens): ",len(ctx_tokens) 
            
            if major == gssc.GSS_S_CONTINUE_NEEDED:
                return 0, ctx_tokens
            elif major == gssc.GSS_S_COMPLETE:
                return 1, ctx_tokens
            else:
                list = gssc.display_status(major, minor)
                ex = apply(GSSContextException, list)
                raise ex

        except GlobusException, ex:
            print "Exception occurred during accept_context test"


    def checkCreateContext(self):
        global g_context_client, g_context_server
        client_cred = GSSCred()
        server_cred = GSSCred()
        client_context = GSSContext()
        server_context = GSSContext()
        client_ctx_created = 0
        server_ctx_created = 0
        server_tokens=""
        while not (client_ctx_created and server_ctx_created):
            if not client_ctx_created:
                client_ctx_created, client_tokens = self.InitContext(client_cred, client_context, server_tokens)
                
                print "client ctx ctx_created", client_ctx_created
            if not server_ctx_created:
                server_ctx_created, server_tokens = self.AcceptContext(server_cred, server_context, client_tokens)
                print "server ctx ctx_created", server_ctx_created
                print len(server_tokens)
               
        t1 = client_context.inquire()
        print t1
        t2 = server_context.inquire()
        print t2
        
        g_context_client = client_context
        g_context_server = server_context

    def checkWrapAndUnwrap(self):
        global g_context_client, g_context_server
        print "checkWrapAndUnwrap"
        msgString = "OZOz0z0"
        wrappedString = g_context_server.wrap(msgString)
        
        unwrappedString = g_context_client.unwrap(wrappedString)
        print unwrappedString
        if msgString == unwrappedString:
            print "Test Passed"
        else:
            print "Test Failed"

        del g_context_client
        del g_context_server   
    def checkGetVerifyMic(self):
        
        print "checkGetAndVerifyMic"
        msgString = "OZOz0z0"
       
        micString = g_context_server.get_mic(msgString)
        
        verified = g_context_client.verify_mic(msgString, micString)
        if verified:
            print "Test Passed"
        else:
            print "Test failed"
            
        
        
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GSSContextTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
    
                  

